# Analysis code for "Evolution of large-scale flow from turbulence in a two-dimensional superfluid"
[(View on GitHub)](https://github.com/shjohnst/BEC-vortices-2D)


## Experiment data notes
The data is available in HDF5 format on [figshare at https://doi.org/10.26180/5bdb8a1059e79](https://doi.org/10.26180/5bdb8a1059e79). The HDF files should be unzipped to a folder on your computer, which is then referenced in your configuration file (see below for details). The example configuration file provided assumes that this data is located in `C:\Experiments\johnstone_vortices_2018`.

Each sequence of shots (see the timestamp in the filenames) corresponds to experiments with a given grid size. There are 25 repeats of 10 different hold times after the grid sweep, which are shuffled in a random order, resulting in 250 shots per sequence.


## Analysis requirements
The analysis code in this repository is written in Python, using the analysis framework of lyse, part of the [labscript suite](http://labscriptsuite.org), an open source experiment control and analysis software.
To install the labscript suite, please use the installer available at https://bitbucket.org/labscript_suite/installer/.
We recommend that you use [Anaconda Python 2.7](https://www.anaconda.com/download/), which includes most of the scientific computing libraries required to run the labscript suite and these analysis scripts. Most of the analysis does not require packages beyond those supplied with Anaconda Python or required by the labscript suite. The vortex detection script, get_vortex_locations_automatically.py, however, requires the OpenCV package `cv2`, which can be installed by running `pip install opencv-python`

The analysis code provided requires lyse version 2.3.1. A list of python package versions used can be found in `python_packages.txt`.

The analysis code also requires the output of the ['MonteCarloSimulations' section of this repository](../MonteCarloSimulations). The required output data is available on [figshare at https://doi.org/10.26180/5bdb871be6ca4](https://doi.org/10.26180/5bdb871be6ca4). This data should be unzipped to a folder on your computer, which is then referenced in your configuration file (see below for details). The example configuration file provided assumes that this data is located in `C:\labscript_suite\userlib\monte_carlo_data\`.

## Configuration file
When setting up the labscript suite, you require a configuration file, which identifies the directories for the experiment shot storage, analysis code, and additionally, in these analysis scripts, the monte carlo data.
This file is automatically generated by the installer, and is located at `C:\labscript_suite\labconfig\<your computer hostname>.ini` (or within the labconfig folder in your labscript suite install directory).
The analysis scripts in this repository require additional custom paths in the configuration file. There is an example configuration file in this folder named `analysis_config.ini` which can be used to replace the default file (rename it to match your host name and replace the existing file).

Alternatively, you can append the paths `simulation_data_storage` and `analysis_output_folder` to your existing configuration file, which correspond to the folders in which you have saved the Monte Carlo data, and where you would like analysis output (figures and spreadsheets) to be saved and change your experiment name to `johnstone_vortices_2018`.

## Analysis workflow
The contents of this folder of the repository should be saved to the analysislib folder defined in your configuration file, typically `C:\labscript_suite\userlib\analysislib\johnstone_vortices_2018`.

The experiment files should be loaded into lyse, which then provides access to a `pandas` dataframe object for the analysis scripts. The order in which analysis scripts were run on the data is outlined below. Note that stages 1-3, which involve running 'single shot' scripts on each experiment file do not need to be re-run in order to run Stage 4, which computes average values across multiple repeats of the experiment and generates figures. **Running scripts in stages 1-3 will overwrite the existing results produced by that script that are saved to each experiment file.**

### Stage 1: Computing optical densities
*Note: running this stage will overwrite the optical density images saved for each experiment.*
* `eigenfringe_OD.py` was run as a single-shot routine in lyse on each experiment as it was run.
* After the first 200 shots of a sequence had been processed, the eigenbasis (which is limited to 200 frames) contains only data from this sequence.
* At this point, those first 200 shots were marked as not done, and `eigenfringe_OD.py` was run on them a second time, so that their OD image could be computed with this complete basis.
* The remaining shots in the sequence were then processed, with the basis adapting to use the last 200 frames.

### Stage 2: Vortex locations
*Note: running scripts in this section will overwrite existing vortex locations saved to the experiment files.*
* `get_vortex_locations_automatically.py` and `review_vortex_locations.py` were run sequentially as single-shot routines in lyse on each experiment shot file.
* `review_vortex_locations.py` requires human intervention to either approve, reject, or adjust the automatically detected vortex locations.

### Stage 3: Vortex signs and individual shot measurements
* The following scripts should be loaded in lyse in this order:
        1. `vortex_signs_classification_and_statistics.py`
        2. `multiprocessing/vortex_signs_classification_and_statistics.py`
        3. `energy_spectra.py`
* The first two scripts will find the sign of each vortex, classify the resulting configuration into clusters, dipoles and free vortices, and perform calculations based on the distribution, including correlation functions and the net dipole moment. There are two versions to help speed up the process - shots with low vortex numbers are calculated with the first, while shots with many vortices use the second which splits the sign detection across multiple processes
* The third script then calculated the incompressible kinetic energy spectrum and total incompressible kinetic energy.

### Stage 4: Multishot analysis
* The scripts in the multishot routine folder can be run either from lyse or directly from a terminal. However, in both cases, the shot files must first be loaded into lyse (which must remain running), which then provides the Pandas DataFrame for the scripts in both cases.  
* The following scrips generate the following figures in "Evolution of large-scale flow from turbulence in a two-dimensional superfluid" and its supplementary material:
    * `plot_multiple_bragg_vortex_images.py`
        * Figure 1: Displays three example shots showing OD, Bragg signal, computed flow field and classified vortex locations.
        
    * `generate_grid_comparison_plots.py`
        * Figure 2: comparing early/late time behaviour across different grids.
        * Supplementary Figure S1, showing early time averaged kinetic energy spectra for each grid.
        
        A spreadsheet containing the output data is available at https://doi.org/10.26180/5bdfb13b96138.
        
    * `generate_time_series_plots_per_grid.py`
        * Figures 4, S3-6: time evolution for a given grid.
        
        A spreadsheet containing the output data is available at https://doi.org/10.26180/5bdfb63136f2c.
        
    * `plot_thermometry_curves.py`:
        * Supplementary Figure S2: showing how the data sits on the appropriate thermometry curves, as well as displaying the un-smoothed curves, and an illustration of how the uncertainty is estimated.
        * Fig. 3: An example of the placement of data on the temperature axis at the position corresponding to the least difference between Monte Carlo curves and the cluster and dipole fractions, and the N_v = 14 panel from Fig. S2.

* Supplementary Fig. S8 was generated using data output from the `vortex_signs_classification_and_statistics.py` single shot routine with an optional flag turned on (see line 292) for the three example cases shown. This data is available at https://doi.org/10.26180/5bdfb87aae28a.

If you have any queries, please contact [Shaun Johnstone](mailto:shaun.johnstone@monash.edu).
